@@ -448,6 +448,11 @@ void CDR_put_short(CDR_Codec *codec, CORBA_short s)
 	CDR_buffer_put2(codec, &s);
 }
 
+CORBA_boolean CDR_get_short(CDR_Codec *codec, CORBA_short *s)
+{
+	return CDR_buffer_get2(codec, s);
+}
+
 void CDR_put_ushort(CDR_Codec *codec, CORBA_unsigned_short us)
 {
 	CDR_buffer_put2(codec, &us);
@@ -463,6 +468,11 @@ void CDR_put_long(CDR_Codec *codec, CORBA_long l)
 	CDR_buffer_put4(codec, &l);
 }
 
+CORBA_boolean CDR_get_long(CDR_Codec *codec, CORBA_long *l)
+{
+	return CDR_buffer_get4(codec, l);
+}
+
 void CDR_put_ulong(CDR_Codec *codec, CORBA_unsigned_long ul)
 {
 	CDR_buffer_put4(codec, &ul);
@@ -474,14 +484,24 @@ CORBA_boolean CDR_get_ulong(CDR_Codec *codec, CORBA_unsigned_long *ul)
 }
 
 #ifdef HAVE_CORBA_LONG_LONG
+CORBA_boolean CDR_get_long_long(CDR_Codec *codec, CORBA_long_long *ul)
+{
+	return CDR_buffer_get8(codec, ul);
+}
+
 void CDR_put_long_long(CDR_Codec *codec, CORBA_long_long ll)
 {
 	CDR_buffer_put8(codec, &ll);
 }
 
-void CDR_put_ulong_long(CDR_Codec *codec, CORBA_unsigned_long_long ull)
+void CDR_put_ulong_long(CDR_Codec *codec, CORBA_unsigned_long_long ll)
 {
-	CDR_buffer_put8(codec, &ull);
+	CDR_buffer_put8(codec, &ll);
+}
+
+CORBA_boolean CDR_get_ulong_long(CDR_Codec *codec, CORBA_unsigned_long_long *ull)
+{
+	return CDR_buffer_get8(codec, ull);
 }
 #endif
 
@@ -520,12 +540,22 @@ void CDR_put_char(CDR_Codec *codec, CORBA_char c)
 	CDR_buffer_put(codec, &c);
 }
 
+CORBA_boolean CDR_get_char(CDR_Codec *codec, CORBA_char *c)
+{
+	return CDR_buffer_get(codec, c);
+}
+
 void CDR_put_boolean(CDR_Codec *codec, CORBA_boolean datum)
 {
 	datum = datum&&1;
 	CDR_buffer_put(codec, &datum);
 }
 
+CORBA_boolean CDR_get_boolean(CDR_Codec *codec, CORBA_boolean *b)
+{
+	return CDR_buffer_get(codec, b);
+}
+
 void CDR_put_string(CDR_Codec *codec, const char *str)
 {
 	unsigned int len;
