@@ -30,6 +30,7 @@
 
 #include "orbit.h"
 #include "orbit_poa.h"
+#include "genrand.h"
 
 PortableServer_ThreadPolicyValue
 PortableServer_ThreadPolicy__get_value(PortableServer_ThreadPolicy  obj, CORBA_Environment *ev)
@@ -341,9 +342,6 @@ PortableServer_POA_create_POA
 	PortableServer_POA new_poa = NULL;
 	PortableServer_POA check_poa = NULL;
 	
-	if(policies)
-		g_warning("non-default POA policies aren't implemented yet");
-
 	/* Check for a child POA by the same name in parent */
 	check_poa = PortableServer_POA_find_POA(poa,adapter_name,
 						FALSE, ev);
@@ -664,7 +662,7 @@ get_objnum_for_obj(PortableServer_POA poa, ORBit_POAObject *obj)
 	if(poa->first_free_id) {
 		retval = poa->first_free_id;
 		poa->first_free_id = GPOINTER_TO_UINT(g_ptr_array_index(poa->objnum_to_obj,
-						       retval));
+									retval));
 		g_ptr_array_index(poa->objnum_to_obj, retval) = obj;
 	} else {
 		retval = poa->objnum_to_obj->len;
@@ -704,8 +702,18 @@ PortableServer_POA_activate_object(PortableServer_POA obj,
 				    NULL);
 		return NULL;
 	}
+	
+	/* Servant Already Active */
+	if((obj->id_uniqueness==PortableServer_UNIQUE_ID) &&
+	   (ORBIT_OBJECT_KEY(servant->_private)->object != 0)) {
+		CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
+				    ex_PortableServer_POA_ServantAlreadyActive,
+				    NULL);
+		return NULL;
+	}
+	
 
-	new_obj = g_new(ORBit_POAObject, 1);
+	new_obj = g_new0(ORBit_POAObject, 1);
 	new_obj->object_id = (PortableServer_ObjectId*)CORBA_sequence_octet__alloc();
 
 	new_objid =
@@ -723,6 +731,7 @@ PortableServer_POA_activate_object(PortableServer_POA obj,
 	new_obj->orb = get_orb_for_poa(obj);
 	new_obj->poa = obj;
 	new_obj->objnum = get_objnum_for_obj(obj, new_obj);
+	orbit_genrand(new_obj->rand_data, ORBIT_RAND_KEY_LEN);
 
 	g_hash_table_insert(obj->active_object_map,
 			    new_obj->object_id,
@@ -757,7 +766,7 @@ PortableServer_POA_activate_object_with_id(PortableServer_POA obj,
 		return;
 	}
 
-	newobj = g_new(ORBit_POAObject, 1);
+	newobj = g_new0(ORBit_POAObject, 1);
 	newobj->object_id = (PortableServer_ObjectId *)CORBA_sequence_octet__alloc();
 	newobj->object_id->_length = id->_length;
 	newobj->object_id->_buffer = CORBA_octet_allocbuf(id->_length);
@@ -766,6 +775,7 @@ PortableServer_POA_activate_object_with_id(PortableServer_POA obj,
 	newobj->poa = obj;
 	newobj->orb = get_orb_for_poa(obj);
 	newobj->objnum = get_objnum_for_obj(obj, newobj);
+	orbit_genrand(newobj->rand_data, ORBIT_RAND_KEY_LEN);
 
 	newobj->servant = p_servant;
 
@@ -834,12 +844,6 @@ PortableServer_POA_create_reference(PortableServer_POA obj,
 	return(NULL);
 }
 
-CORBA_Object PortableServer_POA_create_reference_with_id(PortableServer_POA obj, PortableServer_ObjectId *oid, CORBA_RepositoryId intf, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	return(NULL);
-}
-
 PortableServer_ObjectId *PortableServer_POA_servant_to_id(PortableServer_POA obj, PortableServer_Servant p_servant, CORBA_Environment *ev)
 {
 	PortableServer_ObjectId *retval, *orig;
@@ -856,21 +860,67 @@ PortableServer_ObjectId *PortableServer_POA_servant_to_id(PortableServer_POA obj
 	return retval;
 }
 
-CORBA_Object PortableServer_POA_servant_to_reference(PortableServer_POA obj, PortableServer_Servant p_servant, CORBA_Environment *ev)
+CORBA_Object
+PortableServer_POA_servant_to_reference(PortableServer_POA obj, PortableServer_Servant p_servant, CORBA_Environment *ev)
 {
 	CORBA_Object retval;
-
-	retval = PortableServer_POA_id_to_reference(obj,
-						    ORBIT_OBJECT_KEY(((PortableServer_ServantBase*)p_servant)->_private)->object->object_id, ev);
+	PortableServer_ObjectId *orig_id;
+	PortableServer_ServantBase *servant = p_servant;
+	ORBit_ObjectKey *obj_key =  ORBIT_OBJECT_KEY(servant->_private);
+	
+	int implicit = (obj->implicit_activation == PortableServer_IMPLICIT_ACTIVATION);
+	int activate_able = (obj_key->object == 0) ||
+		(obj->id_uniqueness==PortableServer_MULTIPLE_ID);
+	/* ImplicitActivationPolicy */
+	if( implicit && activate_able) {
+		orig_id = PortableServer_POA_activate_object(obj, p_servant, ev);
+	} else {
+		orig_id = obj_key->object->object_id;
+	}
+	retval = PortableServer_POA_id_to_reference(obj,orig_id,ev);
 
 	return retval;
 }
 
-PortableServer_Servant PortableServer_POA_reference_to_servant(PortableServer_POA obj, CORBA_Object reference, CORBA_Environment *ev)
+PortableServer_Servant
+PortableServer_POA_reference_to_servant(PortableServer_POA obj, CORBA_Object reference, CORBA_Environment *ev)
 {
-	ORBit_POAObject *objinfo;
+	GSList *cur;
 
-	g_assert(!"NYI");
+	g_assert(reference);
+
+	if(obj->request_processing != PortableServer_USE_DEFAULT_SERVANT
+	   && obj->servant_retention != PortableServer_RETAIN) {
+		CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
+				    ex_PortableServer_POA_WrongPolicy,
+				    NULL);
+		return NULL;
+	}
+
+	if(reference->servant)
+		return reference->servant;
+
+	for(cur = reference->profile_list; cur; cur = cur->next) {
+		PortableServer_ObjectId *oid;
+		ORBit_Object_info *curprof = cur->data;
+		ORBit_POAObject *objinfo;
+
+		objinfo = ORBit_POA_find_oid_for_object_key(obj, &(curprof->object_key), &oid);
+		CORBA_free(oid);
+		if(objinfo)
+			return objinfo->servant;
+	}
+
+	CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
+			    ex_PortableServer_POA_ObjectNotActive,
+			    NULL);
+	return NULL;
+}
+
+PortableServer_ObjectId *PortableServer_POA_reference_to_id(PortableServer_POA obj, CORBA_Object reference, CORBA_Environment *ev)
+{
+	PortableServer_ObjectId *retval;
+	ORBit_POAObject *objinfo;
 
 	g_assert(reference);
 	g_assert(reference->active_profile);
@@ -880,31 +930,25 @@ PortableServer_Servant PortableServer_POA_reference_to_servant(PortableServer_PO
 		CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
 				    ex_PortableServer_POA_WrongPolicy,
 				    NULL);
+		return NULL;
 	}
 
-	{
-		CORBA_unsigned_long onum;
-		onum = 
-			*(CORBA_unsigned_long *)(reference->active_profile->object_key._buffer + sizeof(CORBA_unsigned_long));
-		if(onum < obj->objnum_to_obj->len)
-			objinfo = g_ptr_array_index(obj->objnum_to_obj, onum);
-		else
-			objinfo = NULL;
-	}
+	objinfo = ORBit_POA_find_oid_for_object_key(obj, &(reference->active_profile->object_key), &retval);
+	if(objinfo) {
+		CORBA_free(retval);
+		retval = (PortableServer_ObjectId *)CORBA_sequence_octet__alloc();
+		retval->_length = retval->_maximum = objinfo->object_id->_length;
+		retval->_buffer = CORBA_octet_allocbuf(retval->_length);
+		memcpy(retval->_buffer, objinfo->object_id->_buffer, retval->_length);
+		CORBA_sequence_set_release(retval, CORBA_TRUE);
+		return retval;
+	} else if(retval)
+		return retval;
 
-	if(!objinfo) {
-		CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
-				    ex_PortableServer_POA_ObjectNotActive,
-				    NULL);
-	}
-
-	return objinfo->servant;
-}
-
-PortableServer_ObjectId *PortableServer_POA_reference_to_id(PortableServer_POA obj, CORBA_Object reference, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	return(NULL);
+	CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
+			    ex_PortableServer_POA_ObjectNotActive,
+			    NULL);
+	return NULL;
 }
 
 PortableServer_Servant PortableServer_POA_id_to_servant(PortableServer_POA obj, PortableServer_ObjectId *oid, CORBA_Environment *ev)
@@ -931,14 +975,15 @@ PortableServer_Servant PortableServer_POA_id_to_servant(PortableServer_POA obj,
 	return objinfo->servant;
 }
 
-CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
-						PortableServer_ObjectId *oid,
-						CORBA_Environment *ev)
+static CORBA_Object
+my_PortableServer_POA_id_to_reference(PortableServer_POA obj,
+				      PortableServer_ObjectId *oid,
+				      const char *type_id,
+				      CORBA_Environment *ev)
 {
 	GSList *profiles=NULL;
 	ORBit_Object_info *object_info;
 	CORBA_Object retval;
-	CORBA_char *type_id;
 	CORBA_ORB orb;
 	ORBit_POAObject *pobj;
 	ORBit_ObjectKey *objkey = NULL;
@@ -949,9 +994,10 @@ CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
 
 	pobj = g_hash_table_lookup(obj->active_object_map, oid);
 
-	g_assert(pobj);
-	objkey = ORBIT_OBJECT_KEY(((PortableServer_ServantBase *)pobj->servant)->_private);
-	type_id=g_strdup(objkey->class_info->class_name);
+	if(pobj) {
+		objkey = ORBIT_OBJECT_KEY(((PortableServer_ServantBase *)pobj->servant)->_private);
+		type_id= objkey->class_info->class_name;
+	}
 
 	/* Do the local connection first, so it will be attempted first by
 	   the client parsing the IOR string
@@ -962,7 +1008,8 @@ CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
 		object_info->profile_type=IOP_TAG_ORBIT_SPECIFIC;
 		object_info->iiop_major = 1;
 		object_info->iiop_minor = 0;
-		ORBit_POA_find_object_key_for_oid(obj, pobj, &object_info->object_key);
+
+		ORBit_POA_find_object_key_for_oid(obj, pobj, oid, &object_info->object_key);
 
 #ifdef HAVE_IPV6
 		if(orb->cnx.ipv6) {
@@ -984,7 +1031,7 @@ CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
 		object_info->profile_type = IOP_TAG_INTERNET_IOP;
 		object_info->iiop_major = 1;
 		object_info->iiop_minor = 0;
-		ORBit_POA_find_object_key_for_oid(obj, pobj, &object_info->object_key);
+		ORBit_POA_find_object_key_for_oid(obj, pobj, oid, &object_info->object_key);
 
 		object_info->tag.iopinfo.host = g_strdup(IIOP_CONNECTION(orb->cnx.ipv4)->u.ipv4.hostname);
 		object_info->tag.iopinfo.port = ntohs(IIOP_CONNECTION(orb->cnx.ipv4)->u.ipv4.location.sin_port);
@@ -995,13 +1042,12 @@ CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
 
 	retval = ORBit_create_object_with_info(profiles, type_id, orb, ev);
 
-	g_free(type_id);
-
 	if(retval != CORBA_OBJECT_NIL
 	   && ev->_major == CORBA_NO_EXCEPTION
 	   && objkey && objkey->class_info && objkey->class_info->init_local_objref) {
 		/* XXX potential memleak if we get an already-valid objref */
 		retval->vepv = g_new0(gpointer, ORBit_class_assignment_counter + 1);
+		retval->vepv_size = ORBit_class_assignment_counter + 1;
 		objkey->class_info->init_local_objref(retval, pobj->servant);
 		retval->servant = pobj->servant;
 	} else
@@ -1010,6 +1056,21 @@ CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
 	return retval;
 }
 
+CORBA_Object PortableServer_POA_id_to_reference(PortableServer_POA obj,
+						PortableServer_ObjectId *oid,
+						CORBA_Environment *ev)
+{
+	return my_PortableServer_POA_id_to_reference(obj, oid, NULL, ev);
+}
+
+CORBA_Object
+PortableServer_POA_create_reference_with_id(PortableServer_POA obj,
+					    PortableServer_ObjectId *oid,
+					    CORBA_RepositoryId intf,
+					    CORBA_Environment *ev)
+{
+	return my_PortableServer_POA_id_to_reference(obj, oid, intf, ev);
+}
 
 PortableServer_POA PortableServer_Current_get_POA(PortableServer_Current obj, CORBA_Environment *ev)
 {
@@ -1026,7 +1087,6 @@ PortableServer_ObjectId *PortableServer_Current_get_object_id(PortableServer_Cur
 
 CORBA_char *PortableServer_ObjectId_to_string(PortableServer_ObjectId *id, CORBA_Environment *env)
 {
-	g_warning("PortableServer_ObjectId_to_string - FIXME");
 	return CORBA_string_dup((CORBA_char *)id->_buffer);
 }
 
