@@ -167,6 +167,10 @@ ORBit_handle_incoming_message(GIOPRecvBuffer *recv_buffer)
 		giop_recv_buffer_unuse(recv_buffer);
 		giop_main_handle_connection_exception(connection);
 		break;
+	case GIOP_REPLY:
+		/* the above comment probably applies here also... */
+		giop_received_list_push(recv_buffer);
+		break;
 	default:
 		g_warning("discarding message type %d (id possibly %d)",
 			  GIOP_MESSAGE_BUFFER(recv_buffer)->message_header.message_type,
